import { FormBuilder, FormControl } from '@angular/forms';
import { reduce } from 'lodash/fp';

/*
 * FORM UTILITIES
 *
 * Exports
 * -------
 * buildFormGroup(metadata)  - builds FormGroups from metdata, recursively if necessary
 *
 * Variable names
 * --------------
 * metaF   = metadata for Field
 * metaG   = metadata for Group (possibly nested)
 * metaFoG = metadata for Field Or Group
 *
 */

const fb = new FormBuilder();

const addProp = (obj, key, val) => { obj[key] = val; return obj; };

// Helper function to distinguish group from field
const isGroup = metaFoG => metaFoG.meta;

// Adds any missing 'name' properties to Fields and Groups using property's key, recursively
const addNameIfMissing = (metaFoG, key) => metaFoG.name ? metaFoG : addProp(metaFoG, 'name', key);
const addNameToSelfAndChildren = ( [key, metaFoG] ) => {
	metaFoG = addNameIfMissing(metaFoG, key);
	if (isGroup(metaFoG)) {
		metaFoG.meta = addMissingNames(metaFoG.meta); // Recursion
	}
	return [key, metaFoG];
};
const addMissingNames = metaG => Object.entries(metaG)
	.map(addNameToSelfAndChildren)
	.reduce((res, [key, val]) => addProp(res, key, val), {});

// Build Form Control
// TODO: Flesh out function to build validators
const buildControlState = metaF => ({ value: metaF.value || metaF.default, disabled: metaF.isDisabled });
const buildValidators = metaF => ({
	validators: null,
	asyncValidators: null,
	updateOn: 'blur'
});
const buildFormControl = metaF => new FormControl(buildControlState(metaF) /*, buildValidators(metaF) */);

// Build Form Group Member
const buildFormGroupMember = metaFoG => isGroup(metaFoG) ? _buildFormGroup(metaFoG.meta) : buildFormControl(metaFoG);

// Build Form Group
const buildFormGroupReducerIteree = (res, metaFoG) => Object.assign(res, { [metaFoG.name]: buildFormGroupMember(metaFoG) });
const _buildFormGroup = metaG => fb.group(reduce(buildFormGroupReducerIteree, {}, metaG));
const buildFormGroup = metaG => _buildFormGroup(addMissingNames(metaG));

// ---------------------------------------------------------------------------------------------------------------------
// Exports

export { buildFormGroup };
